<?php
/**
 * @package   ats
 * @copyright Copyright (c)2011-2020 Nicholas K. Dionysopoulos / Akeeba Ltd
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\TicketSystem\Site\View\Tickets;

use Akeeba\TicketSystem\Admin\Helper\ComponentParams;
use Akeeba\TicketSystem\Admin\Helper\Credits;
use Akeeba\TicketSystem\Admin\Helper\Permissions;
use Akeeba\TicketSystem\Site\Model\Categories;
use Akeeba\TicketSystem\Site\Model\Tickets;
use JUri;

defined('_JEXEC') or die;

class Html extends \FOF30\View\DataView\Html
{
    /** @var  Categories Category for the current list of tickets */
    protected $category;
    /** @var  bool  Can the user create a ticket? */
    protected $canCreate;
    /** @var  bool  Is the current user a manager? */
    protected $isManager;
    /** @var  array Privileges for the current ticket */
    protected $ticketPerms;

	/**
	 * Should I display a page heading inside the component's output?
	 *
	 * @var   bool
	 * @since 3.0.0
	 */
    public $showPageHeading = false;

	/**
	 * Joomla menu item ID
	 *
	 * @var   int
	 * @since 3.0.0
	 */
	public $itemID;

	/**
	 * The state of the "Status" filter
	 *
	 * @var   string
	 * @since 3.0.0
	 */
    public $filterStatus;

	/**
	 * Current user
	 *
	 * @var   \JUser
	 * @since 3.0.0
	 */
	public $user;

	/**
	 * Ticket owner user
	 *
	 * @var   \JUser
	 * @since 3.0.0
	 */
	public $ownerUser;

	/**
	 * The full name of the user the ticket is assigned to (or the translated "Unassigned" string)
	 *
	 * @var   string
	 * @since 3.0.0
	 */
	public $assignedUserFullName;

	/**
	 * Should I display credits information?
	 *
	 * @var   bool
	 * @since 3.0.0
	 */
	public $showCredits;

	/**
	 * How many credits are required to file a reply?
	 *
	 * @var   int
	 * @since 3.0.0
	 */
	public $creditsRequired;

	/**
	 * Does the user have enough credits?
	 *
	 * @var   bool
	 * @since 3.0.0
	 */
	public $enoughCredits;

	/**
	 * Total amount of credits charged to the user for this ticket only
	 *
	 * @var   int
	 * @since 3.0.0
	 */
	public $creditsCharged;

	/**
	 * Total amount of credits charged to the user on all of their tickets
	 *
	 * @var   int
	 * @since 3.0.0
	 */
	public $totalCredits;

	/**
	 * Return URL, after submitting the form, base64-encoded
	 *
	 * @var   string
	 * @since 3.0.0
	 */
	public $returnURL;

	/**
	 * Is Time Spent a mandatory field?
	 *
	 * @var   bool
	 * @since 3.0.0
	 */
	public $timeSpentIsMandatory;

	/**
	 * Results of plugins responding to the onBeforeTicketConversationDisplay event
	 *
	 * @var   array
	 * @since 3.0.0
	 */
	public $beforeConversationResults;

	/**
	 * Results of plugins responding to the onAfterTicketConversationDisplay event
	 *
	 * @var   array
	 * @since 3.0.0
	 */
	public $afterConversationResults;

	/**
	 * The partial URL for actions to be performed by a manager
	 *
	 * @var   string
	 * @since 3.0.0
	 */
	public $adminActionsURL;

	/**
	 * The URL for the Print View of a ticket
	 *
	 * @var   string
	 * @since 3.0.0
	 */
	public $printURL;

	public $creditsLeft;

	public $creditsRequiredTicket;

	public $enableUserFeedback;

	protected function onBeforeRead()
	{
		parent::onBeforeRead();

		/** @var \JApplicationSite $app */
		$app              = \JFactory::getApplication();
		$params           = $app->getParams();
		$this->pageParams = $params;

		/** @var Tickets $ticket */
		$ticket = $this->item;

		$this->ticketPerms = Permissions::getPrivileges($ticket);
		$this->isManager   = Permissions::isManager($ticket->catid);

		$this->container->platform->importPlugin('ats');

		// Push the Joomla user model into the view. In this way we can always reference the same
		// object (and cache its data) instead of keep creating it and running extra queries
		$juserModel = $this->container->factory->model('Jusers');
		$this->setModel('Jusers', $juserModel);

		$this->user                      = $this->container->platform->getUser();
		$this->ownerUser                 = $this->container->platform->getUser($ticket->created_by);
		$this->showCredits               = ComponentParams::getParam('showcredits', 0);
		$this->creditsRequired           = Credits::creditsRequired($ticket->catid, false, $ticket->public, $ticket->priority);
		$this->creditsRequiredTicket     = Credits::creditsRequired($ticket->catid, true, $ticket->public, $ticket->priority);
		$this->enoughCredits             = Credits::haveEnoughCredits($this->user->id, $ticket->catid, false, $ticket->public, $ticket->priority);
		$this->totalCredits              = Credits::creditsCharged('all', $ticket->ats_ticket_id);
		$this->creditsCharged            = Credits::creditsCharged('ticket', $ticket->ats_ticket_id);
		$this->creditsLeft               = Credits::creditsLeft($ticket->created_by);
		$this->returnURL                 = base64_encode(JUri::getInstance()->toString());
		$this->timeSpentIsMandatory      = (int) ($this->container->params->get('timespent_mandatory', 0) && $this->isManager && ATS_PRO);
		$this->beforeConversationResults = $this->container->platform->runPlugins('onBeforeTicketConversationDisplay', [$ticket]);
		$this->afterConversationResults  = $this->container->platform->runPlugins('onAfterTicketConversationDisplay', [$ticket]);
		$this->itemID                    = $this->input->getInt('Itemid', 0);
		$this->assignedUserFullName      = $ticket->assigned_to ? $this->getContainer()->platform->getUser($ticket->assigned_to)->name : \JText::_('COM_ATS_TICKETS_UNASSIGNED');
		$this->adminActionsURL           = 'index.php?option=com_ats&view=Ticket&id=' . $ticket->ats_ticket_id . '&returnurl=' . $this->returnURL . '&Itemid=' . $this->itemID . '&' . $this->getContainer()->platform->getToken(true) . '=1';
		$this->enableUserFeedback = ComponentParams::getParam('userfeedback', 0);

		$this->printURL = clone JURI::getInstance();
		$this->printURL->setVar('layout', 'print');
		$this->printURL->setVar('tmpl', 'component');


		if ($this->isManager)
		{
			// Javascript required for enabling tabs
			$this->addJavascriptFile('media://com_ats/js/tabs_enable.js', $this->getContainer()->mediaVersion);
		}

		$timeSpentIsMandatoryJS = $this->timeSpentIsMandatory ? 'true' : 'false';
		$ticketJS = <<< JS

;//
var ats_timespent_mandatory = $timeSpentIsMandatoryJS;

JS;
		$this->addJavascriptInline($ticketJS);
	}

	protected function onBeforePrint()
    {
        $this->onBeforeRead();
    }

    protected function onBeforeBrowse()
    {
	    /** @var Tickets $model */
	    $model = $this->getModel();

	    // Apply custom filters for frontend
	    $model->frontendfilter(1)
		    ->enabled(1)
		    ->filterNewest(1)
		    ->filter_order_Dir('DESC');

	    parent::onBeforeBrowse();

	    // Let's fetch the category from the request or page params
	    $category_id = $this->input->getInt('category', 0);

	    if (!($category_id))
	    {
		    // Maybe I got here directly from the menu
		    $category_id = $this->pageParams->get('category', 0);
	    }

	    /** @var Categories $category */
	    $category       = $this->container->factory->model('Categories')->tmpInstance();
	    $this->category = $category->category($category_id)->get()->first();

	    $actions         = Permissions::getAclPrivileges($this->category->id);
	    $this->canCreate = $actions['core.create'];

	    $this->isManager = Permissions::isManager($this->category->id);

	    $this->showPageHeading = $this->getPageParams()->get('show_page_heading', 1);
	    $this->filterStatus    = $this->getModel()->getState('status', '');
	    $this->itemID          = $this->input->getInt('Itemid', 0);
    }
}
